function [alphamc,Smc,IMSmc,LIMSmc,sepsmc,lambmc,postseps,postlamb,etamc,etaMSmc,etapostmc,etapostMSmc,anmc,ancholmc,likMSmc,dlo_miss] = mixture3mcmc_neu(y,ZG,ZMS,miss,dlo,dlo_out,indexb,indexn,index0,isp_lag_dlo,lag_dir,maxlag,IMS,S,pralm,pralinf,Q,prseps,seps,prlamb,lamb,unit_spec_var,time_spec_var,K,eta,etaMS,alpha,perm,e0,e0MS,frac,M,it0,it0cov,it0b)
%[alphamc,bmc,Qmc,Qinvmc,Qinvdet,postQS,postQnu,Smc,IMSmc,sepsmc,postseps,etamc,etaMSmc,etapostmc,etapostMSmc,anmc,ancholmc,dlo_miss] = mixture3mcmc_neu(y,ZG,ZMS,miss,dlo,dlo_out,indexb,indexn,index0,isp_lag_dlo,IMS,pralm,pralinf,prQnu,prQS,Q,prseps,seps,K,eta,etaMS,alpha,perm,e0,e0MS,frac,M,it0,it0cov,it0b)
% latent class mixed effect modell

% die Matrix ZG besteht aus einem variablen und einem fixen Teil
% ZG(:,1:dd,:) .. variabel .. X2
% ZG(:,dd+1:end,:) .. fix .. X1
% die Matrix ZMS enthaelt spalten von ZG, welche ueber die Zeit einen gruppenspezifischen, wechselnden (switchenden) Einfluss haben
% ZMS(:,1:dMS,:) 


% Modell: y = X1*alpha + X2*D(1)*betak(1) + .. + X2*D(K)*betak(K) + X3*D(K)betaR(K)*(I_t^K,-1) + seps  fuer alle i (Index weggelassen)
% mit K .. Anzahl an Klassen
% D(k) .. Dummy fuer Klassenzugehrigkeit
% seps - N(0,sigmaeps^2*I)
% b(i) - N(0,Q(i)); Q(i) = Q(:,:,S(i)); S(i).. gibt Gruppenzug an, vorlauefig nur pooling Q=0.

% input: 
%    - observations and design matrices:

%        y (T times N)  observations
%           T .. number of repetions
%           N .. number of objects

%        ZG (T times dd+1:end times N) design matrix for the fixed effects
%           d-dd .. dimension of  parameter alpha of fixed effects

%        ZG (T times 1:dd times N) design matrix for random effects
%           dd .. dimension of  parameter betak of random effects

%        ZMS (T times 1:dMS times N) design matrix for switching effects
%           dMS .. dimension of  parameter betaR of switching effects

%        IMS (K times T) matrix of group-specific state variables, wenn size(IMS,1)==1, dann gleiches IMS fuer alle Gruppen (mit gruppenspezifischen switching effects)
%           

%    - prior information:

%        pralm  (d+dMS times 1)/(d+dd*(k-1)+dMS*K) ... normal prior distribution of fixed effect:  mean 
%        pralinf (d+dMS times d+dMS)/(d+dd*(k-1)+dMS*K times d+dd*(k-1)+dMS*K) ... normal prior distribution of fixed effect: information matrix 
%                                                                                                                                                                       (inverse of covariance)
%                        bei dim d noch aufblasen in Programm entspechend auskommentieren

%
%        prQ  .. inverted Wishart prior distribution of Q, equal for all groups
%             prQnu ... degrees of freedom (scalar)
%             prQS  ... scale matrix (dd times dd)
%        prseps (1 times 2).. inverted gamma prior distribution of seps
%             1,1 ... degrees of freedom
%             1,2 ... scale parameter

%
%        e0 (1 times K) .. Dirichlet prior for eta
%        e0MS  Dirichlet prior for etaMS (2 times 2 times K) if group-specific transition probabilities,
%                                        (2 times 2) if group-independent transition probabilities

%    - Size and starting value for MCMC:

%        M ... size of MCMC-sample
%        it0 ... burn-in 
%        it0b ... M-it0b values are saved for Smc and IMSmc
%        it0cov ... M-it0cov values are saved for ancm and ancholmc
%        sp=M-it0;
%        spcov=fix(M-it0cov);
%        spb=min(100,M-it0b);


%        Starting value:

%          seps (scalar) variance of the observation equation
%          Q (dd times dd times K) symmetric covariance matrix of the random effects
%          alpha (d+dd*(K-1)+dMS*K) fixed effects and group specific effects
%          eta (K) weights
%          etaMS (2 times 2 times K) if group-specific transition probabilities,
%                (2 times 2) if group-independent transition probabilities
%          IMS (K times T) group-specific state variable

%    - Model:
%          perm ... to define the restriction for the permutation sampler
%          K ... to define the number of groups

% output:
%        MCMC-sample:
%          alphamc (sp times d+dd*(k-1)+dMS*K)  group specific, fixed and group-specific switching effects
%          sepsmc (sp times 1) variance of the observation equation
%          etamc (sp times K) weigths
%          etaMSmc (size(e0MS) times sp) transition probabilities
%          Smc (spb times N) group indicator
%          IMSmc (K times N times spb) state variable

%        Parameters of conditional densities used for estimation (if post==1): 
%          postQS(spcov times dd*(dd+1)/2 times K) scale matrix of inverted  Wishart posterior of Q
%          postseps(spcov)  scale parameter of inverted gamma posterior densities of seps
%          etapostmc(spcov times K)
%          anmc(spcov times d+dd*(k-1)+dMS*K)
%          ancolmc(spcov times d+dd*(k-1)+dMS*K times d+dd*(k-1)+dMS*K)

test_IMS_FIX=0; 
%test_IMS_FIX=1; perm=0;%FLAG DAMIT IMS FIX BLEIBT 


post=1; %flag to save the parameters of posteriors
d = size(ZG,2); % davon sind die ersten dd Spalten variabel
dd = size(Q,1);
dMS=size(ZMS,2); %zahl der spalten, die in der Zeit switchen

N = size(y,2);
T = size(y,1);
lag_dlo=size(isp_lag_dlo,2);
% nobs_nu=(lag_dir-lag_dlo)*(lag_dir>lag_dlo);
nobs_nu=maxlag-lag_dlo;
dlo_miss=zeros(size(dlo));

dlo_m=median(dlo);dlo_s=iqr(dlo);      
nst=size(etaMS,1);
states = 1:nst;
groups = [1:K]';
etaMSmc=zeros(2,2,size(etaMS,3),1);
etapostMSmc=zeros(2,2,size(etaMS,3),1);
IMSmc=zeros(size(IMS,1),size(IMS,2));
LIMSmc=(zeros(size(IMS,1),size(IMS,2),1)==0);
likMSmc=zeros(nst,K,size(IMS,2),1);
lik=zeros(nst,K,size(IMS,2),1);

sp=M-it0;
spcov=fix(M-it0cov);
spb=min(100,M-it0b);

%Z=zeros(T,dd*(K-1)+d+dMS*K,N);      % aufgeblasene Matrix aus ZG: erste dd*K Spalten fuer die K Gruppen,letzte (d-K*dd) Spalten fuer die fixen Effekte
alphamc=zeros(sp,dd*(K-1)+d+dMS*K);  % dimension entprechend zu Z passend
sepsmc=zeros(sp,1);
lambmc=lamb(1:max(~time_spec_var+time_spec_var*T),:);
if unit_spec_var
    lambmc=zeros(~time_spec_var+time_spec_var*T,N,spcov); % nur jeden r-ten schritt wird abgespeichert (um platz zu sparen), der index
                                                           % fuer das entsprechende sepsmc wird in post_ind gespeichert.
end
etamc=zeros(sp,K);
if dMS>0 
    etaMSmc=zeros(nst,nst,size(etaMS,3),sp);
    if post
        etaMSpostmc=zeros(nst,nst,size(etaMS,3),spcov);
        likMSmc=zeros(nst,K,size(IMS,2),spcov);
    else
        likMSmc=zeros(size(IMS,1),K,size(IMS,2),1);
    end
    IMSmc=zeros(size(IMS,1),size(IMS,2),spb);
    LIMSmc=(zeros(size(IMS,1),size(IMS,2),sp)==0);

    index_col=[1:size(ZG,2)];
    index_perm2=zeros(dMS,1);
    for iMS=1:dMS
        incol=index_col(all(all((ZG-ZMS(:,iMS*ones(size(ZG,2),1),:))==0,3)==1,1));
        %%incol=1;'TEST prior in mixture3mcmc_neu'
        if isempty(incol) 
            ['warning: column ' num2str(iMS) ' of ZMS not contained in ZG']
        else   
            index_perm2(iMS)=incol;
        end  
    end   
    %search_colMS; % bestimmt index_perm2, gibt an in welchen spalten von ZG sich die switching variablen befinden;
else
    etaMSmc=[]; etaMSpostmc=[]; IMSmc=[];
end

shilf=1;
it0mmc=M;rat=M;
postseps=zeros(1,2);
postlamb=zeros(1,2);
etapostmc=zeros(1,K);
anmc=zeros(1,dd*(K-1)+d+dMS*K);
ancholmc=zeros(1,dd*(K-1)+d+dMS*K,dd*(K-1)+d+dMS*K);

if post
    % fr Speicherung von anmc, ancholmc fr jeden rat-ten Schritt ab Stationaritt:
    if spcov > 0
        rat = floor((M-it0)/spcov);
        %         for it = it0:M,
        %             if (M-it)/spcov == rat
        %                 it0mmc=it;
        %             end
        %         end   
        it = it0:M;
        it0mmc=it(find(((M-it)/spcov)==rat));
    end   
    
    postseps=zeros(spcov,2);
%     if unit_spec_var
%         postlamb=zeros(spcov,2,N);
%     end
    etapostmc=zeros(spcov,K);
    anmc=zeros(spcov,dd*(K-1)+d+dMS*K);
    ancholmc=zeros(spcov,dd*(K-1)+d+dMS*K,dd*(K-1)+d+dMS*K);
end

% Qinv=zeros(size(Q));
% detQ=zeros(1,K);
% Qq=zeros(size(qincol(Q(:,:,1))));
% Qqinv=Qq;
% QS=zeros(dd,dd,K);
% Qnu=0;
% if any(any(squeeze(any(Q)))) == 1
%     spQ=sp; spcovQ=spcov; spbQ=spb;
% else  
%     spQ=1; spcovQ=1; spbQ=1;
% end
% 
% Qmc=zeros(spQ,dd*(dd+1)/2,K);             
% Qinvmc=zeros(spQ,dd*(dd+1)/2,K);             
% Qinvdet=zeros(spQ,K);             
% 
% if post
%     postQS=zeros(spcovQ,dd*(dd+1)/2,K);
%     postQnu=zeros(spcovQ,K);
% end
% 
% bmc=zeros(spbQ,N,dd);

eps = zeros(T,N); 
%S=zeros(1,N); %defined in daten_read_b
Smc=zeros(spb,N);
% probmc=zeros(spb,K,N);
prob = zeros(K,N);
%Vinv = zeros(T,T,N); %nur zeros(dd,dd,K) sind noetig, im aufbau von ANinv und a beruecksichtigen.
% Kk = zeros(dd,T,N);
% B = zeros(dd,dd,N);
% b = zeros(N,dd);
%yb = zeros(T,N);
y_mf=zeros(T,N);
y_MS=zeros(T,N);

sp=0;  % zur Berechnung der Likelihood

indexG = kron(ones(1,K),[1:dd]); %reshape(index1(ones(1,K),:)',1,K*dd);
indexK = kron([1:K]',ones(dd,1))'; %reshape(states(:,ones(1,dd))',1,K*dd);
indexR = kron(ones(1,K),[1:dMS]);
indexM = kron([1:K]',ones(dMS,1))';
indexMIX=[1:dd*K];
indexFIX=[K*dd+1:d+(K-1)*dd];
indexMS=[(d+(K-1)*dd+1):(d+(K-1)*dd+dMS*K)];


for m=1:M
    
    if any(m==[1:1000:M]) 
        m
    end
    
    ANinv = pralinf;
    a = pralinf*pralm;
    
    % 3. sampling the group indicator
    
    if K>1
    [S,sp, Vinv] = simstateexmix_new(y,eta,ZG,ZMS,IMS,Q,alpha,indexMIX,indexFIX,indexMS,seps,lamb,dd,sp);  
else
    S=ones(size(S));
      Vinv=lamb./seps;
  end
    
    D = S(ones(K,1),:)==groups(:,ones(N,1)); %group dummy
    
    %   1a.  Sampling the fixed and group-specific (non-random) effects from the marginal model
    if dMS>0;
        if size(IMS,1)<K
            IMStr=IMS(ones(K,1),:)';
        else
            IMStr=IMS';
        end
    end
    for k=1:K;
        %X'X (gruppenweise berechnet) zeilenweise abspeichern (block fuer die gruppenspezifischen, fuer die fixen, fuer die
        % gruppenspezifisch switchenden
        
        Xtr=reshape(permute([ZG(:,:,S==k).*permute(sqrt(Vinv(:,S==k,ones(d,1))),[1 3 2])],[2 1 3]),d,sum(S==k)*T);
        if sum(S==k)==0;
           Xtr=zeros(d+dMS,1);
        end
        if (dMS>0)*(sum(S==k)>0);
            Xtr=[Xtr; reshape(permute([(ZMS(:,:,S==k).*permute(sqrt(Vinv(:,S==k,ones(dMS,1))),[1 3 2])) ... 
                        .*(permute([IMStr(:,k,ones(dMS,1),ones(sum(S==k),1))-1],[1 3 4 2]))],[2 1 3]),dMS,sum(S==k)*T)]; %X-transpose
         end
         
         XX = Xtr*Xtr';
         if sum(S==k)>0;
            Xy = Xtr*reshape([y(:,S==k).*sqrt(Vinv(:,S==k))],size(Xtr,2),1);
         elseif sum(S==k)==0;
            Xy= zeros(d+dMS,1);
         end
                 
        ANinv((k-1)*dd+[1:dd],[(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) = ANinv((k-1)*dd+[1:dd],[(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) + XX(1:dd,:);        
        ANinv(indexFIX,[(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) = ANinv(indexFIX,[(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) + XX(dd+1:d,:);
        ANinv(indexMS((k-1)*dMS+[1:dMS]), [(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) = ANinv(indexMS((k-1)*dMS+[1:dMS]), [(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) + XX(d+1:end,:);
        a([(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])]) = a([(k-1)*dd+[1:dd] indexFIX indexMS((k-1)*dMS+[1:dMS])])+ Xy;
    end
    
    % schur decomposition   - informationsmatrix
    
    [U, Td]=schur(ANinv);
    Td=diag(1./(abs(diag(real(Td))).^.5));
    U=real(U);
    %   ancov=U*Td.^2*U';
    an = U*Td.^2*U'*a;
    anchol = U*Td;
    alpha = an + anchol*randn(dd*(K-1)+d+dMS*K,1);
    
    %  4. sampling the group probability
    
    [eta,etapost] = simtransex(D,e0);
    if dMS>0 
        [etaMS,etapostMS] = simxi_ms(IMS'+1,e0MS);
    end
    
    
    %alph=permute(alpha([(S(ones(dd,1),:)'-1)*dd+kron(ones(N,1),[1:dd]) kron(ones(N,1),indexFIX) reshape([indexMIX([(S(ones(dMS,1),:)'-1)*dMS+kron(ones(N,1),[1:dMS])])],N,dMS)]),[3 2 1])
    alph=permute(alpha([(S(ones(dd,1),:)'-1)*dd+kron(ones(N,1),[1:dd]) kron(ones(N,1),indexFIX)]),[3 2 1]);
    if N==1
        alph=reshape(alph,1,size(alph,3),1);
    end
    y_mf = squeeze(sum(ZG.*alph(ones(T,1),:,:),2));
    if dMS>0 
        IMSmat=reshape(IMStr(:,S),T,1,N);
        alph=permute(alpha([reshape([indexMS([(S(ones(dMS,1),:)'-1)*dMS+kron(ones(N,1),[1:dMS])])],N,dMS)]),[3 2 1]);
        if N==1
            alph=reshape(alph,1,size(alph,3),1);
        end
        y_MS = squeeze(sum( (ZMS.*(IMSmat(:,ones(1,dMS),:)-1)).*alph(ones(T,1),:,:),2));
        eps = y - y_mf - y_MS;
    else
        eps = y - y_mf;
    end
    
    
    
    %  2b. variances of observation equation   
    
%     posts = prseps + .5*frac*[N*T  sum(sum(eps.^2,1).*lamb(1,:),2)];
    posts = prseps + .5*frac*[N*T  sum(sum((eps.*sqrt(lamb)).^2,1),2)];
    % 'TEST VARIANCE: prior=post'     posts=prseps;
    %%%%%
%     gamrnd('state',m);
%     seps_prev = 1./gamrnd(posts_prev(1,1),1./posts_prev(1,2));
%     gamrnd('state',m);
    seps = 1./gamrnd(posts(1,1),1./posts(1,2));
    
    % sample unit-specific lambda's
    if and(unit_spec_var,~time_spec_var)
%         postl = prlamb(ones(N,1),:)./2 + 0.5*[T*ones(N,1) (sum(eps.^2,1)./seps)'];
        postl = prlamb(ones(N,1),:)./2 + 0.5*[T*ones(N,1) (sum((eps./sqrt(seps)).^2,1))'];
%         gamrnd('state',m);
%         lamb_prev = kron(gamrnd(postl_prev(:,1),1./postl_prev(:,2))',ones(T,1));
%         gamrnd('state',m);
        lamb = kron(gamrnd(postl(:,1),1./postl(:,2))',ones(T,1));
    end
    if and(unit_spec_var,time_spec_var)
        %         postl = prlamb(ones(N,1),:)./2 + 0.5*[T*ones(N,1) (sum(eps.^2,1)./seps)'];
        postl = prlamb(ones(T*N,1),:)./2 + 0.5*[ones(T*N,1) (eps(:).^2)./seps(:)];
        %         gamrnd('state',m);
        %         lamb_prev = kron(gamrnd(postl_prev(:,1),1./postl_prev(:,2))',ones(T,1));
        %         gamrnd('state',m);
        lamb = reshape(gamrnd(postl(:,1),1./postl(:,2)),T,N);
    end

    

    
    % Sample MS-Indicator
    
    if dMS>0
        Dstat1=permute(D,[3 1 2]);
        ystat1 = squeeze(sum((-ZMS(:,indexR,:).*Dstat1(ones(size(ZMS,1),1),kron(1:K,ones(1,dMS)),:)).*permute(alpha([indexMS],ones(size(ZMS,1),1),ones(size(ZMS,3),1)),[2 1 3]),2));
        lik = full_lik_new([y-y_mf-ystat1],[y-y_mf],S,seps./lamb,size(IMS,1));
        %%TEST IMS fix: folgenden Code nicht ausfhren
        if ~test_IMS_FIX
            
            IMS = simstate_ms_new(etaMS,lik)'-1;
        end
        
    end
    
    if miss==1
        % sample missing values
        Tall=size(dlo,1)-nobs_nu;
        dlo_use=dlo(1+nobs_nu:end,:);
        for i=1:N
            ib=indexb(i);
            imiss=dlo_out(1+nobs_nu:end,ib)'==1;
            if any(imiss)
                betai=[alpha((S(i)-1)*dd+[1:dd]);alpha(indexFIX);zeros(dMS,1)];
                betai=betai(:,ones(1,T));
                if dMS>0
                    betai(d+1:d+dMS,:)=betai(d+1:d+dMS,:)+alpha(indexMS([(S(i)-1)*dMS+[1:dMS]]),ones(1,T)).*(permute([IMS(S(i),:,ones(dMS,1))],[3,2,1])-1);
                end   
                
                if lag_dlo>0
                    isp_nolag=[1:(isp_lag_dlo(1)-1) (isp_lag_dlo(end)+1):size(ZG,2)];
                    %construct Bi
                    Bi=reshape([betai(isp_lag_dlo(end:-1:1),:)' -ones(T,1) zeros(T)]',T*(Tall+1),1);
                else
                    isp_nolag=[1:size(ZG,2)];
                    Bi=reshape([-ones(T,1) zeros(T)]',T*(Tall+1),1); %richtig?
                end   
                Bi=reshape(Bi(1:end-T),Tall,T)';
                izero=all(Bi(:,imiss)==0,2);
                Bitilde=Bi(~izero,imiss)./sqrt(seps./lamb(~izero,i*ones(sum(imiss),1)));
                dc=Bi(~izero,~imiss)*dlo_use(~imiss,ib)+sum(squeeze(ZG(~izero,isp_nolag,i)).*betai(isp_nolag,~izero)',2);
                if dMS>0; %THIS WAS NOT IN THE ORIGINAL PROGRAMS!
                    dc=dc+sum(squeeze(ZMS(~izero,:,i)).*betai(d+1:end,~izero)',2);
                end
                dc=dc./sqrt(seps./lamb(~izero,i));
                C0=(1.34/5)^2*diag((1./dlo_s(ib*ones(1,size(Bitilde,2)))).^2);
                Sy=inv(Bitilde'*Bitilde+C0);                                           
                my=Sy*(-Bitilde'*dc+C0*dlo_m(ib*ones(1,size(Bitilde,2)))');
                
                ytilde=my+chol(Sy)'*randn(size(my,1),1);
                dlo_use(imiss,ib)=ytilde;
            end   
            
        end   
        dlo(1+nobs_nu:end,:)=dlo_use;
        % impute missing values
        y=dlo(indexn,indexb);
        if lag_dlo>0
            for i1=1:lag_dlo
                ZG(1:size(indexn,1),isp_lag_dlo(i1),1:size(indexb,2))=dlo(indexn-i1,indexb);
            end
        end   
    end   
    
    
    % 5. Permutation
    
    if perm == 100
        si = randperm(K); 
        %si=[1:K];
        siMS = randperm(1+(dMS>0));
        %siMS=[1:2];  %keine permutation 
    else
        si =[1:K];siMS=[1:2];
    end
    
    %CHANGE THE FOLLOWING IF FIXED EFFECTS ARE ALSO SWITCHING
    alphai=reshape(alpha([indexMIX indexMS]),dd,K,1+(dMS>0)); 
    
    if and(dMS>0,siMS(1)==2)
        % match group specific and fixed parameters 
        iiMIX=index_perm2(index_perm2<=dd);
        iiFIX=index_perm2(index_perm2>dd)+(K-1)*dd; 
        
        %CHANGE THE FOLLOWING IF FIXED EFFECTS ARE ALSO SWITCHING
        alphai(iiMIX,:,1)=alphai(iiMIX,:,1)-alphai(iiMIX,:,2);
        %alpha(iiFIX)=alpha(iiFIX)-alpha(indexMS(???));  %CHOOSE WHICH STATE VARIABLE IS RELEVANT FOR FIXED EFFECTS
        alphai(iiMIX,:,2)=-alphai(iiMIX,:,2);
        IMS = 1-IMS;
        etaMS=etaMS(siMS,siMS,:);
        alpha(indexMIX,1) = reshape(alphai(:,:,1),dd*K,1);
        alpha(indexMS,1) = reshape(alphai(:,:,2),dd*K,1);
        lik=lik(siMS,:,:);

    end
    
    if any(si~=[1:K])
        [si1,ssort]=sort(si);
        S = ssort(S);
        %CHANGE THE FOLLOWING IF FIXED EFFECTS ARE ALSO SWITCHING
        alpha(indexMIX,1) = reshape(alphai(:,si,1),dd*K,1);
        if dMS>0         alpha(indexMS,1) = reshape(alphai(:,si,2),dd*K,1); end
        eta = eta(si);
         lik=lik(:,si,:);
        %Q = Q(:,:,si); 
        %Qinv=Qinv(:,:,si);
        %detQ=detQ(1,si);
        if size(IMS,1)>1;
            IMS = IMS(si,:);
        end
        if size(etaMS,3)>1;
            etaMS=etaMS(:,:,si);
        end
    end
    
    
    % 6. abspeichern
    if m > it0 
        alphamc(m-it0,:)=alpha';
        sepsmc(m-it0)=seps;
%         sepsmc_prev(m-it0)=seps_prev;
%         if unit_spec_var
%             lambmc(:,:,m-it0)=lamb(1:max(~time_spec_var,time_spec_var*T),:);
% %             lambmc_prev(:,:,m-it0)=lamb_prev(1:max(~time_spec_var,time_spec_var*T),:);
%         end
        etamc(m-it0,:)=eta;
        if dMS>0 
            etaMSmc(:,:,:,m-it0)=etaMS; 
             LIMSmc(:,:,m-it0)=(IMS==1); 
        end
        dlo_miss=dlo_miss+dlo;
    end
    
    if (m == it0mmc + rat*shilf)*post
        anmc(shilf,:)=an';
        ancholmc(shilf,:,:)=anchol;
        postseps(shilf,:)=posts;
        if unit_spec_var
            lambmc(:,:,shilf)=lamb(1:max(~time_spec_var,time_spec_var*T),:);
%             lambmc_prev(:,:,m-it0)=lamb_prev(1:max(~time_spec_var,time_spec_var*T),:);
        end
        etapostmc(shilf,:)=etapost;
        if dMS>0 
            etapostMSmc(:,:,:,shilf)=etapostMS; 
           likMSmc(:,:,:,shilf)=lik;
       end
        shilf = shilf+1;
    end
    
    if m > M-spb
        Smc(m-(M-spb),:)=S; 
        if dMS>0  IMSmc(:,:,m-(M-spb))=IMS; end
    end
    
end 


dlo_miss=dlo_miss/(M-it0);