function cp = prior_IN_log(Imc,xi)




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input: 

% Imc ... path of the indicator sampled from the posterior (T times M);
% comments:
% 1.  Imc(t,m) refers to the value of process m at time t

% xi ... transition matirx

% Comments:
% 1. If xi is an L x L array  (meaning that size(xi,3)=1),
%    then xi is  the same for all processes 

% 2. otherwise xi is a L x L x M with M=size(xi,3)>1,
%    then xi is  different for the various processes 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output:

%cp .. log of the prior of Imc given xi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Sylvia Fruehwirth-Schnatter
% Last change: 3. Juli 2003
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

M=size(Imc,2);
L=size(xi,1);

   
cN=zeros(L,L,M);
cp=0;
   
   for m=1:M
      cN(:,:,m) = countrns(Imc(:,m)',L) ;
   end

if size(xi,3)==1
   %  xi is assumed to be the same for all processes 
   
   for m=1:M
      cp = cp + sum(sum(squeeze(cN(:,:,m)).*log(xi)));
   end
   
   
elseif size(xi,3)==M
   %  xi is assumed to be different for the various processes 
   
    for m=1:M
      cp = cp + sum(sum(squeeze(cN(:,:,m)).*log(squeeze(xi(:,:,m)))));
   end
   
 
else
   'xi and M do not agree in priori_IN_log'
end  

